@@ -39,7 +39,6 @@ logger = getLogger()
 class Ted2Zim:
     def __init__(
         self,
-        max_videos_per_topic,
         topics,
         debug,
         name,
@@ -88,7 +87,6 @@ class Ted2Zim:
             if not topics
             else [c.strip().replace(" ", "+") for c in topics.split(",")]
         )
-        self.max_videos_per_topic = max_videos_per_topic
         self.autoplay = autoplay
         self.playlist = playlist
         self.subtitles_enough = subtitles_enough
@@ -132,6 +130,7 @@ class Ted2Zim:
             [] if not self.languages else self.to_ted_langcodes(self.languages)
         )
         self.zim_lang = None
+        self.already_visited = []
 
     @property
     def templates_dir(self):
@@ -209,13 +208,13 @@ class Ted2Zim:
                     self.append_part1_or_part3(lang_code_list, lang_info)
         return list(set(lang_code_list))
 
-    def extract_videos_from_playlist(self):
+    def extract_videos_from_playlist(self, playlist):
         """ extracts metadata for all videos in the given playlist
 
             calls extract_video_info on all links to get this data
         """
 
-        playlist_url = f"{self.playlists_base_url}/{self.playlist}"
+        playlist_url = f"{self.playlists_base_url}/{playlist}"
         soup = BeautifulSoup(download_link(playlist_url).text, features="html.parser")
         video_elements = soup.find_all("a", attrs={"class": "hover/appear"})
         self.playlist_title = soup.find("h1", attrs={"class": "f:4"}).string
@@ -224,63 +223,58 @@ class Ted2Zim:
         for element in video_elements:
             relative_path = element.get("href")
             url = urllib.parse.urljoin(self.talks_base_url, relative_path)
-            if self.extract_video_info(url):
-                if self.source_languages:
+            if self.extract_info_from_video_page(url):
+                if self.source_languages and len(self.source_languages) > 1:
                     other_lang_urls = self.generate_urls_for_other_languages(url)
                     logger.debug(
                         f"Searching info for the video in other {len(other_lang_urls)} language(s)"
                     )
                     for lang_url in other_lang_urls:
-                        self.extract_video_info(lang_url)
+                        self.extract_info_from_video_page(lang_url)
+                    self.already_visited.append(urllib.parse.urlparse(url)[2])
             logger.debug(f"Seen {relative_path}")
         logger.debug(f"Total videos found on playlist: {len(video_elements)}")
         if not video_elements:
             raise ValueError("Wrong playlist ID supplied. No videos found")
 
-    def extract_videos_from_topics(self):
-        """ extracts metadata for required number of videos on different topics """
-        for topic in self.topics:
-            logger.debug(f"Fetching video links for topic: {topic}")
-            topic_url = f"{self.talks_base_url}?topics%5B%5D={topic}"
-            total_videos_scraped = 0
-            video_allowance = self.max_videos_per_topic
+    def generate_search_result_and_scrape(self, topic_url, total_videos_scraped):
+        """ generates a search result and returns the total number of videos scraped """
 
-            if self.source_languages:
-                for lang in self.source_languages:
-                    topic_url = topic_url + f"&language={lang}"
-                    page = 1
-                    while video_allowance:
-                        html = download_link(f"{topic_url}&page={page}").text
-                        num_videos_extracted = self.extract_videos_on_topic_page(
-                            html, video_allowance,
-                        )
-                        if num_videos_extracted == 0:
-                            break
-                        video_allowance -= num_videos_extracted
-                        total_videos_scraped += num_videos_extracted
-                        page += 1
-            else:
-                page = 1
-                while video_allowance:
-                    html = download_link(f"{topic_url}&page={page}").text
-                    num_videos_extracted = self.extract_videos_on_topic_page(
-                        html, video_allowance,
-                    )
-                    if num_videos_extracted == 0:
-                        break
-                    video_allowance -= num_videos_extracted
-                    total_videos_scraped += num_videos_extracted
-                    page += 1
-            logger.info(f"Total video links found in {topic}: {total_videos_scraped}")
-            if total_videos_scraped == 0:
-                self.topics.remove(topic)
-                logger.debug(f"Removed topic {topic} from list as it had no videos")
-        if not self.topics:
-            if self.source_languages:
-                raise ValueError(
-                    "No videos found for any topic in the language(s) requested. Check topic(s) and/or language code supplied to --languages"
+        page = 1
+        while True:
+            html = download_link(f"{topic_url}&page={page}").text
+            nb_videos_extracted, nb_videos_on_page = self.extract_videos_on_topic_page(
+                html
+            )
+            if nb_videos_on_page == 0:
+                break
+            total_videos_scraped += nb_videos_extracted
+            page += 1
+        return total_videos_scraped
+
+    def extract_videos_from_topics(self, topic):
+        """ extracts metadata for required number of videos on different topics """
+
+        logger.debug(f"Fetching video links for topic: {topic}")
+        topic_url = f"{self.talks_base_url}?topics%5B%5D={topic}"
+        total_videos_scraped = 0
+
+        if self.source_languages:
+            for lang in self.source_languages:
+                topic_url = topic_url + f"&language={lang}"
+                total_videos_scraped = self.generate_search_result_and_scrape(
+                    topic_url, total_videos_scraped
                 )
-            raise ValueError("Wrong topic(s) were supplied. No videos found")
+
+        else:
+            total_videos_scraped = self.generate_search_result_and_scrape(
+                topic_url, total_videos_scraped
+            )
+
+        logger.info(f"Total video links found in {topic}: {total_videos_scraped}")
+        if total_videos_scraped == 0:
+            return False
+        return True
 
     def update_zim_metadata(self):
 
@@ -290,9 +284,13 @@ class Ted2Zim:
             if len(self.source_languages) > 1:
                 self.zim_lang = "mul"
             else:
-                self.zim_lang = get_language_details(
+                lang_info = get_language_details(
                     self.source_languages[0], failsafe=True
-                )["iso-639-3"]
+                )
+                if lang_info:
+                    self.zim_lang = lang_info["iso-639-3"]
+                else:
+                    self.zim_lang = "eng"
 
         if self.playlist:
             if not self.title:
@@ -306,20 +304,19 @@ class Ted2Zim:
                 if not self.description:
                     self.description = "A selection of TED videos from several topics"
             else:
+                topic_str = self.topics[0].replace("+", " ")
                 if not self.title:
-                    topic_str = self.topics[0].replace("+", " ")
                     self.title = f"{topic_str.capitalize()} from TED"
                 if not self.description:
                     self.description = f"A selection of {topic_str} videos from TED"
 
     def get_display_name(self, lang_code, lang_name):
         """ Display name for language """
-        if lang_code != "en":
-            return (
-                get_language_details(lang_code, failsafe=True)["native"]
-                + " - "
-                + lang_name
-            )
+
+        lang_info = get_language_details(lang_code, failsafe=True)
+        if lang_code != "en" and lang_info:
+
+            return lang_info["native"] + " - " + lang_name
         return lang_name
 
     def get_subtitle_dict(self, lang):
@@ -339,14 +336,16 @@ class Ted2Zim:
             "languageCode": lang["languageCode"],
         }
 
-    def generate_subtitle_list(self, video_id, langs, page_lang):
+    def generate_subtitle_list(self, video_id, langs, page_lang, audio_lang):
         """ List of all subtitle languages with link to their pages """
 
         subtitles = []
         if self.subtitles_setting == ALL or (not self.source_languages and self.topics):
             subtitles = [self.get_subtitle_dict(lang) for lang in langs]
         elif self.subtitles_setting == MATCHING or (
-            self.subtitles_enough and self.subtitles_setting == NONE
+            self.subtitles_enough
+            and self.subtitles_setting == NONE
+            and page_lang != audio_lang
         ):
             subtitles = [
                 self.get_subtitle_dict(lang)
@@ -385,7 +384,7 @@ class Ted2Zim:
                 urls.append(urllib.parse.urlunparse(url_parts))
         return urls
 
-    def extract_videos_on_topic_page(self, page_html, video_allowance):
+    def extract_videos_on_topic_page(self, page_html):
 
         # all videos are embedded in a <div> with the class name 'row'.
         # we are searching for the div inside this div, that has an <a>-tag
@@ -395,27 +394,22 @@ class Ted2Zim:
         soup = BeautifulSoup(page_html, features="html.parser")
         video_links = soup.select("div.row div.media__image a")
         nb_extracted = 0
-        logger.debug(f"{str(len(video_links))} video(s) found on current page")
+        nb_listed = len(video_links)
+        logger.debug(f"{nb_listed} video(s) found on current page")
         for video_link in video_links:
             url = urllib.parse.urljoin(self.talks_base_url, video_link["href"])
-            if not [
-                video
-                for video in self.videos
-                if video.get("tedpath", None) == video_link["href"]
-            ]:
-                if self.extract_video_info(url):
-                    nb_extracted += 1
-                    if self.source_languages:
-                        other_lang_urls = self.generate_urls_for_other_languages(url)
-                        logger.debug(
-                            f"Searching info for video in other {len(other_lang_urls)} language(s)"
-                        )
-                        for lang_url in other_lang_urls:
-                            self.extract_video_info(lang_url)
-                    if nb_extracted == video_allowance:
-                        break
-                logger.debug(f"Seen {video_link['href']}")
-        return nb_extracted
+            if self.extract_info_from_video_page(url):
+                nb_extracted += 1
+                if self.source_languages and len(self.source_languages) > 1:
+                    other_lang_urls = self.generate_urls_for_other_languages(url)
+                    logger.debug(
+                        f"Searching info for video in other {len(other_lang_urls)} language(s)"
+                    )
+                    for lang_url in other_lang_urls:
+                        self.extract_info_from_video_page(lang_url)
+                    self.already_visited.append(urllib.parse.urlparse(url)[2])
+            logger.debug(f"Seen {video_link['href']}")
+        return nb_extracted, nb_listed
 
     def get_lang_code_from_url(self, url, with_full_query=False):
         """ gets the queried language code from a ted talk url """
@@ -430,36 +424,74 @@ class Ted2Zim:
             return current_lang, query
         return current_lang
 
-    def extract_video_info(self, url):
-        """ extract all info from a TED video url and updates self.videos """
-
-        # Extract the meta-data of the video:
-        # Speaker, the profession of the speaker, a short biography of
-        # the speaker, the link to a picture of the speaker, title,
-        # publishing date, view count, description of the TED talk,
-        # direct download link to the video, download link to the subtitle
-        # files and a link to a thumbnail of the video.
-        # Every TED video page has a <script>-tag with a Javascript
-        # object with JSON in it. We will just stip away the object
-        # signature and load the json to extract meta-data out of it.
-        # returns True if successfully scraped new video
-        soup = BeautifulSoup(download_link(url).text, features="html.parser")
-        div = soup.find("div", attrs={"class": "talks-main"})
-        script_tags_within_div = div.find_all("script")
-        if len(script_tags_within_div) == 0:
-            logger.error("The required script tag containing video meta is not present")
-            return False
-        json_data_tag = script_tags_within_div[-1]
-        json_data = json_data_tag.string
-        json_data = json.loads(json_data[18:-1])["__INITIAL_DATA__"]
-        lang_code = json_data["language"]
-
-        requested_lang_code = self.get_lang_code_from_url(url)
-        if requested_lang_code and lang_code != requested_lang_code:
-            logger.error(
-                f"Video has not yet been translated into {requested_lang_code}"
+    def update_videos_list(
+        self,
+        video_id,
+        lang_code,
+        lang_name,
+        title,
+        description,
+        speaker,
+        speaker_profession,
+        speaker_bio,
+        speaker_picture,
+        date,
+        thumbnail,
+        video_link,
+        length,
+        subtitles,
+    ):
+        # append to self.videos and return if not present
+        if not [video for video in self.videos if video.get("id", None) == video_id]:
+            self.videos.append(
+                {
+                    "id": video_id,
+                    "languages": [
+                        {
+                            "languageCode": lang_code,
+                            "languageName": self.get_display_name(lang_code, lang_name),
+                        }
+                    ],
+                    "title": [{"lang": lang_code, "text": title}],
+                    "description": [{"lang": lang_code, "text": description}],
+                    "speaker": speaker,
+                    "speaker_profession": speaker_profession,
+                    "speaker_bio": speaker_bio,
+                    "speaker_picture": speaker_picture,
+                    "date": date,
+                    "thumbnail": thumbnail,
+                    "video_link": video_link,
+                    "length": length,
+                    "subtitles": subtitles,
+                }
             )
-            return False
+            logger.debug(f"Successfully inserted video {video_id} into video list")
+            return True
+
+        # update localized meta for video if already in self.videos
+        # based on --subtitles=matching
+        logger.debug(f"Video {video_id} already present in video list")
+        for index, video in enumerate(self.videos):
+            if video.get("id", None) == video_id:
+                if {"lang": lang_code, "text": title} not in video["title"]:
+                    self.videos[index]["title"].append(
+                        {"lang": lang_code, "text": title}
+                    )
+                    self.videos[index]["description"].append(
+                        {"lang": lang_code, "text": description}
+                    )
+                    self.videos[index]["languages"].append(
+                        {
+                            "languageCode": lang_code,
+                            "languageName": self.get_display_name(lang_code, lang_name),
+                        }
+                    )
+                if self.subtitles_setting == MATCHING or self.subtitles_setting == NONE:
+                    self.videos[index]["subtitles"] += subtitles
+        return False
+
+    def extract_video_info_from_json(self, json_data):
+        lang_code = json_data["language"]
         lang_name = json_data["requested_language_english_name"]
         talk_info = json_data["talks"][0]
         native_talk_language = talk_info["player_talks"][0]["nativeLanguage"]
@@ -492,31 +524,16 @@ class Ted2Zim:
             else:
                 speaker = "None"
 
-        # Extract the profession of the speaker of the TED talk
+        # Extract the ted talk details from json
+        video_id = talk_info["id"]
         speaker_profession = speaker_info["description"]
-
-        # Extract the short biography of the speaker of the TED talk
         speaker_bio = speaker_info["whotheyare"]
-
-        # Extract the Url to the picture of the speaker of the TED talk
         speaker_picture = speaker_info["photo_url"]
-
-        # Extract the title of the TED talk
         title = talk_info["title"]
-
-        # Extract the description of the TED talk
         description = talk_info["description"]
-
-        # Extract the upload date of the TED talk
         date = dateutil.parser.parse(talk_info["recorded_at"]).strftime("%d %B %Y")
-
-        # Extract the length of the TED talk in minutes
         length = int(talk_info["duration"]) // 60
-
-        # Extract the thumbnail of the of the TED talk video
         thumbnail = talk_info["player_talks"][0]["thumb"]
-
-        # Extract the download link of the TED talk video
         download_links = talk_info["downloads"]["nativeDownloads"]
         if not download_links:
             logger.error("No direct download links found for the video")
@@ -526,67 +543,70 @@ class Ted2Zim:
             logger.error("No link to download video in medium quality")
             return False
 
-        # Extract the video Id of the TED talk video.
-        # We need this to generate the subtitle page.
-        video_id = talk_info["id"]
-
         langs = talk_info["player_talks"][0]["languages"]
-        subtitles = self.generate_subtitle_list(video_id, langs, lang_code)
+        subtitles = self.generate_subtitle_list(
+            video_id, langs, lang_code, native_talk_language
+        )
+        return self.update_videos_list(
+            video_id=video_id,
+            lang_code=lang_code,
+            lang_name=lang_name,
+            title=title,
+            description=description,
+            speaker=speaker,
+            speaker_profession=speaker_profession,
+            speaker_bio=speaker_bio,
+            speaker_picture=speaker_picture,
+            date=date,
+            thumbnail=thumbnail,
+            video_link=video_link,
+            length=length,
+            subtitles=subtitles,
+        )
 
-        # Extract the keywords for the TED talk
-        keywords = soup.find("meta", attrs={"name": "keywords"})["content"]
-        keywords = [key.strip() for key in keywords.split(",")]
+    def extract_info_from_video_page(self, url, retry_count=0):
+        """ extract all info from a TED video page url and update self.videos """
 
-        # append to self.videos and return if not present
-        if not [video for video in self.videos if video.get("id", None) == video_id]:
-            self.videos.append(
-                {
-                    "id": video_id,
-                    "languages": [
-                        {
-                            "languageCode": lang_code,
-                            "languageName": self.get_display_name(lang_code, lang_name),
-                        }
-                    ],
-                    "title": [{"lang": lang_code, "text": title}],
-                    "description": [{"lang": lang_code, "text": description}],
-                    "speaker": speaker,
-                    "speaker_profession": speaker_profession,
-                    "speaker_bio": speaker_bio,
-                    "speaker_picture": speaker_picture,
-                    "date": date,
-                    "thumbnail": thumbnail,
-                    "video_link": video_link,
-                    "length": length,
-                    "subtitles": subtitles,
-                    "keywords": keywords,
-                    "tedpath": urllib.parse.urlparse(url)[2],
-                }
+        # Every TED video page has a <script>-tag with a Javascript
+        # object with JSON in it. We will just stip away the object
+        # signature and load the json to extract meta-data out of it.
+        # returns True if successfully scraped new video
+
+        # don't scrape if URL already visited
+        if urllib.parse.urlparse(url)[2] in self.already_visited:
+            return False
+
+        # don't scrape if maximum retry count is reached
+        if retry_count > 5:
+            logger.error("Max retries exceeded. Skipping video")
+            return False
+
+        soup = BeautifulSoup(download_link(url).text, features="html.parser")
+        div = soup.find("div", attrs={"class": "talks-main"})
+
+        # TED is sometimes inconsistant in sending HTML content
+        # it sometimes sends the HTML without the required div containing the talks data
+        # so we retry after 5 seconds
+        if not div:
+            logger.debug(
+                "Potentially insufficient data returned by server. Retrying in 5 seconds..."
             )
-            logger.debug(f"Successfully inserted video {video_id} into video list")
-            return True
+            time.sleep(5)
+            return self.extract_info_from_video_page(url, retry_count=retry_count + 1)
 
-        # update localized meta for video if already in self.videos
-        # based on --subtitles=matching
-        logger.debug(f"Video {video_id} already present in video list")
-        for index, video in enumerate(self.videos):
-            if video.get("id", None) == video_id:
-                if {"lang": lang_code, "text": title} not in video["title"]:
-                    self.videos[index]["title"].append(
-                        {"lang": lang_code, "text": title}
-                    )
-                    self.videos[index]["description"].append(
-                        {"lang": lang_code, "text": description}
-                    )
-                    self.videos[index]["languages"].append(
-                        {
-                            "languageCode": lang_code,
-                            "languageName": self.get_display_name(lang_code, lang_name),
-                        }
-                    )
-                if self.subtitles_setting == MATCHING:
-                    self.videos[index]["subtitles"] += subtitles
-        return False
+        script_tags_within_div = div.find_all("script")
+        if len(script_tags_within_div) == 0:
+            logger.error("The required script tag containing video meta is not present")
+            return False
+        json_data = script_tags_within_div[-1].string
+        json_data = json.loads(json_data[18:-1])["__INITIAL_DATA__"]
+        requested_lang_code = self.get_lang_code_from_url(url)
+        if requested_lang_code and json_data["language"] != requested_lang_code:
+            logger.error(
+                f"Video has not yet been translated into {requested_lang_code}"
+            )
+            return False
+        return self.extract_video_info_from_json(json_data)
 
     def add_default_language(self):
         """ add metatada in default language (english or first avail) on all videos """
@@ -644,18 +664,14 @@ class Ted2Zim:
         env = jinja2.Environment(
             loader=jinja2.FileSystemLoader(str(self.templates_dir)), autoescape=True
         )
-        languages = []
-        for video in self.videos:
-            for language in video["subtitles"]:
-                languages.append(
-                    {
-                        "languageCode": language["languageCode"],
-                        "languageName": language["languageName"],
-                    }
-                )
-            languages += video["languages"]
+        all_langs = {
+            language["languageCode"]: language["languageName"]
+            for video in self.videos
+            for language in video["subtitles"] + video["languages"]
+        }
         languages = [
-            dict(tpl) for tpl in set(tuple(item.items()) for item in languages)
+            {"languageName": value, "languageCode": key}
+            for key, value in all_langs.items()
         ]
         languages = sorted(languages, key=lambda x: x["languageName"])
         html = env.get_template("home.html").render(languages=languages)
@@ -849,6 +865,18 @@ class Ted2Zim:
         logger.info(f"uploaded {video_path} to cache at {key}")
         return True
 
+    def remove_failed_topics_and_check_extraction(self, failed_topics):
+        """ removes failed topics from topics list and raises error if scraper cannot continue """
+
+        for topic in failed_topics:
+            self.topics.remove(topic)
+        if not self.topics:
+            if self.source_languages:
+                raise ValueError(
+                    "No videos found for any topic in the language(s) requested. Check topic(s) and/or language code supplied to --languages"
+                )
+            raise ValueError("Wrong topic(s) were supplied. No videos found")
+
     def run(self):
         logger.info(
             f"Starting scraper with:\n"
@@ -864,10 +892,18 @@ class Ted2Zim:
                 f"Using cache: {self.s3_storage.url.netloc} with bucket: {self.s3_storage.bucket_name}"
             )
 
+        # playlist mode requested
         if self.playlist:
-            self.extract_videos_from_playlist()
+            self.extract_videos_from_playlist(self.playlist)
+        # topic(s) mode requested
         else:
-            self.extract_videos_from_topics()
+            failed = []
+            for topic in self.topics:
+                if not self.extract_videos_from_topics(topic):
+                    failed.append(topic)
+                else:
+                    logger.debug(f"Successfully scraped {topic}")
+            self.remove_failed_topics_and_check_extraction(failed)
 
         self.add_default_language()
         self.update_zim_metadata()
